##################################################################################
#
#   Kaivo Public Software License
#
#   Copyright (c) 2001, Kaivo, Inc.
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions are met:
#
#       o Redistributions of source code must retain the above copyright notice,
#         this list of conditions and the following disclaimer.
#
#       o Redistributions in binary form must reproduce the above copyright
#         notice, this list of conditions and the following disclaimer
#         in the documentation and/or other materials provided with the
#         distribution.
#
#       o Neither Kaivo nor the names of its contributors may be used to endorse
#         or promote products derived from this software without specific prior
#         written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#   ARE DISCLAIMED. IN NO EVENT SHALL KAIVO OR ITS CONTRIBUTORS BE LIABLE FOR 
#   ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
#   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
#   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND 
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
#   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##################################################################################
"""MSWord doc content munger"""

from os import listdir, popen, remove, path
from string import find, rfind
from Globals import package_home, DTMLFile
from OFS.DTMLMethod import DTMLMethod
from OFS.DTMLDocument import DTMLDocument
from time import time

tmp_path = '/tmp'
wvConf_file = path.join(package_home(globals()), 'wvZope.xml')

# Uses the wvWare, and assumes it is in the path
# Requires write access to /tmp

manage_MSWordMungerAddForm = DTMLFile('dtml/MSWordMungerAdd', globals())

def wordWrap(text, width):
    remaining = text
    wrapped = ''

    while len(remaining) > width:
        cut = width

        if remaining[cut] != ' ':
            cut = rfind(remaining, ' ', 0, cut-1)
            if cut == -1: cut = find(remaining, ' ', cut-1)
            if cut == -1: cut = len(remaining)

        wrapped = wrapped + remaining[:cut] + '\n'
        remaining = remaining[cut+1:]

    if remaining:
        wrapped = wrapped + remaining

    return wrapped

def convert(data, width=80):
    """Convert MSWord data to simple DTML"""
    
    tmp_name = ''

    while (not tmp_name) or (tmp_name in listdir(tmp_path)):
        # Pick a unique name for the tmp file
        tmp_name = tmp_path + '/munger_tmp_' + str(time())
    
    tmp_file = open(tmp_name, 'wb')
    tmp_file.write(data)
    tmp_file.close()
    dtmllines = popen('wvWare -x %s %s 2> /dev/null' % (wvConf_file, tmp_name)).readlines()
    remove(tmp_name)

    # Clean up and word wrap the output

    dtml=''
    for line in dtmllines:
        dtml = dtml + wordWrap(line, width)
    
    return dtml

# For increased speed, comment out line 400 of text.c in the wv source

def manage_addMSWordMunger(self, id, title='', file='', method=1, REQUEST=None, RESPONSE=None):
    """Create a dtml object from a Word doc"""

    if hasattr(file, 'read'):
        file = file.read()

    if not file:
        raise 'EmptyFile', 'No data to munge!'

    dtml = convert(file)

    if not dtml:
        raise 'ConversionError', 'Converter returned no data.'

    id = str(id)

    if method:
        ob = DTMLMethod(dtml, __name__ = id)
    else:
        ob = DTMLDocument(dtml, __name__ = id)

    ob.title = str(title)
    ob.content_type = 'text/html'
    self._setObject(id, ob)

    if RESPONSE:
        RESPONSE.setCookie('LastMungedToAMethod', str(method), path='/')
        RESPONSE.redirect(self.absolute_url() + '/manage_main')
    else:
        return self._getOb(id)    
